"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class CodeGenerator {
    constructor() {
        this.indentWidth = 2;
        this.indentLevel = 0;
        this.startOfIndentLevel = true;
        this.output = '';
    }
    print(maybeString) {
        if (maybeString) {
            this.output += maybeString;
        }
    }
    printEmptyLine() {
        if (/\n\n$/.test(this.output)) {
            return;
        }
        else if (/\n$/.test(this.output)) {
            this.print('\n');
        }
        else {
            this.print('\n\n');
        }
    }
    printNewline() {
        if (this.output) {
            this.print('\n');
            this.startOfIndentLevel = false;
        }
    }
    printOnNewline(maybeString) {
        if (maybeString) {
            if (!/\n$/.test(this.output)) {
                this.printNewline();
            }
            this.printIndent();
            this.print(maybeString);
        }
    }
    printAndForceInline(content) {
        this.print(content);
    }
    printIndent() {
        const indentation = ' '.repeat(this.indentLevel * this.indentWidth);
        this.output += indentation;
    }
    withIndent(closure) {
        this.indentLevel += 1;
        closure();
        this.indentLevel -= 1;
    }
    withinBlock(closure) {
        this.print('{');
        this.withIndent(closure);
        this.output = this.output.replace(/\s*$/, '');
        this.printOnNewline('}');
    }
}
exports.default = CodeGenerator;
